<?php

/**
 * This add-on allows for a module:
 * - To specify structure and default values of all module settings in one place: "hook_get_settings()".
 * - Use variable naming standards: <module_name> "_" <setting_name>.
 * - Use defined default values everywhere (with option to override default value)
 *   Example:
 *     Instead of "variable_get('my_module_var1', 456)" in several places of a code, you can just use "setting_get('my_module', 'var1')".
 *     To override default value use the same approach as for "variable_get": "setting_get('my_module', 'var1', 987)".
 * - Automatic check for existence of setting at the moment of "set" or "get" and in case if setting it's not defined raise an error.
 * - You will never forget about to delete some variable at "hook_uninstall()", because you can just call "setting_delete_all('my_module')".
 * - Automatic specify default values ('#default_value') for admin settings form by using "setting_prepare_settings_form('my_module', $form)",
 *   this will also automatically rename element names from format '<setting_name>' to format '<module_name> "_" <setting_name>'
 *   (for proper processing with "system_settings_form_submit()").
 */

function setting_get_settings($module) {
  global $settings;

  if (!isset($settings[$module])) {
    //$settings[$module] = module_invoke($module, 'get_settings');
    $settings[$module] = call_user_func($module . '_' . 'get_settings'); // for use at bootstrap
    if (!isset($settings[$module])) {
      $settings[$module] = array();
    }
  }

  return $settings[$module];
}

function setting_check($module, $name) {
  $module_settings = setting_get_settings($module);

  if (isset($module_settings[$name])) {
    return TRUE;
  }
  else {
    trigger_error(t('Setting %name in module %module is not exist.', array('%name' => $name, '%module' => $module)), E_USER_WARNING);
  }
}

function setting_get($module, $name, $value = NULL) {
  if (setting_check($module, $name)) {
    if (!isset($value)) {
      $module_settings = setting_get_settings($module);
      $value = $module_settings[$name];
    }
    return variable_get($module.'_'.$name, $value);
  }
}

function setting_set($module, $name, $value) {
  if (setting_check($module, $name)) {
    variable_set($module.'_'.$name, $value);
  }
}

function setting_get_all($module, $values) {
  global $conf;

  $result = array();
  $module_settings = setting_get_settings($module);
  foreach ($module_settings as $name => $default_value) {
    $result[$name] = (array_key_exists($module.'_'.$name, $conf) ? $conf[$module.'_'.$name] : (array_key_exists($name, $values) ? $values[$name] : $default_value));
  }
  return $result;
}

function setting_reset_all($module) {
  $module_settings = setting_get_settings($module);
  foreach ($module_settings as $name => $value) {
    variable_set($module.'_'.$name, $value);
  }
}

function setting_delete_all($module) {
  $module_settings = setting_get_settings($module);
  foreach (array_keys($module_settings) as $name) {
    variable_del($module.'_'.$name);
  }
}

//TODO: use "element_children()" (?)
function setting_prepare_settings_form($module, $form) {
  $result_form = $form;
  foreach ($form as $name => $element) {
    // If this is a form element
    if (($name{0} != '#') && is_array($element)) {
      // If this is an input form element
      if (in_array($element['#type'], array('checkbox', 'checkboxes', 'date', 'file', 'password', 'radio', 'radios', 'select', 'textarea', 'textfield', 'weight', 'hidden'))) {
        // If "default_value" applicable
        if (!in_array($element['#type'], array('file', 'password'))) {
          $element['#default_value'] = setting_get($module, $name);
        }

        // Rename element name in result form
        $result_form[$module.'_'.$name] = $element;
        unset($result_form[$name]);
      }
      else {
        $result_form[$name] = setting_prepare_settings_form($module, $element);
      }
    }
  }
  return $result_form;
}
